#include "BucketsNode.h"

/**
 * BucketsNode.cc - Search node for buckets puzzle.  Problem: Given
 * a 5 unit and a 3 unit bucket, how can one measure precisely 4
 * units?  Possible operations: fill/empty 1st/2nd bucket or pour
 * contents of one to the other until source empty or recipient is
 * full.
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */

/**
 * Creates a <code>BucketsNode</code> instance and sets it
 * to an initial search state. */
BucketsNode::BucketsNode() {
  bucket1 = 0;
  bucket2 = 0;
}

/**
 * <code>isGoal</code> - test whether or not the current node is a
 * goal node.
 *
 * @return a <code>boolean</code> value */
bool BucketsNode::isGoal() 
{
  return (bucket1 + bucket2) == 4;
}


/**
 * <code>expand</code> - return a (possibly empty) vector of
 * pointers to this node's children
 *
 * @return a pointer to a <code>vector</code> of
 * <code>SearchNode</code> pointers */
vector<SearchNode *> *BucketsNode::expand() 
{
  // For each operation, check if it is valid and will change
  // the state.  If so, generate a child and add it to the
  // children vector.
  vector<SearchNode *> *children = new vector<SearchNode *>;
  
  // Empty bucket1
  if (bucket1 > 0) {
    BucketsNode *child = (BucketsNode *) childClone();
    child->bucket1 = 0;
    children->push_back(child);
  }
  
  // Empty bucket2
  if (bucket2 > 0) {
    BucketsNode *child = (BucketsNode *) childClone();
    child->bucket2 = 0;
    children->push_back(child);
  }
  
  // Fill bucket1
  if (bucket1 != MAX_AMOUNT1) {
    BucketsNode *child = (BucketsNode *) childClone();
    child->bucket1 = MAX_AMOUNT1;
    children->push_back(child);
  }
  
  // Fill bucket2
  if (bucket2 != MAX_AMOUNT2) {
    BucketsNode *child = (BucketsNode *) childClone();
    child->bucket2 = MAX_AMOUNT2;
    children->push_back(child);
  }   
  
  // Pour bucket1 into bucket2
  if (bucket2 < MAX_AMOUNT2 && bucket1 > 0) {
    BucketsNode *child = (BucketsNode *) childClone();
    int pourAmount = MAX_AMOUNT2 - bucket2;
    if (bucket1 < pourAmount)
      pourAmount = bucket1;
    child->bucket1 -= pourAmount;
    child->bucket2 += pourAmount;
    children->push_back(child);
  }
  
  // Pour bucket2 into bucket1
  if (bucket1 < MAX_AMOUNT1 && bucket2 > 0) {
    BucketsNode *child = (BucketsNode *) childClone();
    int pourAmount = MAX_AMOUNT1 - bucket1;
    if (bucket2 < pourAmount)
      pourAmount = bucket2;
    child->bucket2 -= pourAmount;
    child->bucket1 += pourAmount;
    children->push_back(child);
  }
  
  return children;
}

/**
 * <code>deepCopy</code> - return a deep copy of this node.
 *
 * @return an <code>SearchNode *</code> value
 */
SearchNode *BucketsNode::deepCopy() 
{
  return new BucketsNode(*this);
}


/**
 * <code>toStream</code> - print a text representation of this search
 * node to the given output stream. */
ostream& BucketsNode::toStream(ostream &sout) 
{
  sout << "Bucket(" << MAX_AMOUNT1 << "): " << bucket1 << ", Bucket(" 
       << MAX_AMOUNT2 << "): " << bucket2;
  return sout;
}


/**
 * == - Equality overloading is especially useful for repeated state
 * detection.
 *
 * @param n1 a <code>BucketsNode</code> value - first node
 * @param n2 a <code>BucketsNode</code> value - second node
 * @return a <code>bool</code> value - whether or not the two nodes are equal
 */
bool operator==(BucketsNode n1, BucketsNode n2) 
{
  return (n1.bucket1 == n2.bucket1) && (n1.bucket2 == n2.bucket2);
}

