#ifndef LIGHTS_OUT_NODE_H
#include <vector>
#include <stdlib.h>
#include <time.h>
#include "SearchNode.h"
using namespace std;

/**
 * class <code>LightsOutNode</code> - a puzzle where one seeks to get
 * all lights out in a grid of lights.  Each move turns a light
 * on/off.  However, this also toggles horizontally/vertically
 * adjacent lights on/off.  The puzzle starts in its solved state.
 * Take a number of random moves to scramble it and see if you can
 * return it to the solved state!
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */


class LightsOutNode : public SearchNode {

 public:

    /**
     * Creates a new <code>LightsOutNode</code> instance with a light
     * grid of the given size.  Also seeds the random number generator
     * with the system time.
     *
     * @param size an <code>int</code> value - size of light grid */
    LightsOutNode(int size);


    /**
     * Destructor freeing dynamically allocated array.
     */
    ~LightsOutNode();
    

    /**
     * <code>isGoal</code> - tests whether or not all lights are out
     *
     * @return a <code>boolean</code> value - whether or not all lights are out
     */
    bool isGoal();

    
    /**
     * <code>expand</code> - return a (possibly empty) vector of
     * pointers to this node's children
     *
     * @return a pointer to a <code>vector</code> of
     * <code>SearchNode</code> pointers */
    vector<SearchNode *> *expand();

    
    /**
     * <code>makeMove</code> - toggle the light at the given index
     * (row * size + column) along with all lights adjacent
     * orthogonally in the grid.
     *
     * @param index an <code>int</code> value - light row * size + column */
    void makeMove(int index);


    /**
     * <code>makeRandomMove</code> - make a single random move
     */
    void makeRandomMove();
    
    
    /**
     * <code>makeRandomMoves</code> - make <code>moves</code> random moves
     *
     * @param moves an <code>int</code> value - number of random moves
     * to make */
    void makeRandomMoves(int moves);
    

    /**
     * <code>deepCopy</code> - return a deep copy of this node.
     *
     * @return an <code>SearchNode *</code> value
     */
    SearchNode *deepCopy();

    
    /**
     * <code>toStream</code> - print a text representation of this search
     * node to the given output stream. */
    ostream& toStream(ostream &sout);

    // You can (and should) add your own public/private variables to
    // keep track of state.

};


/**
 * == - Equality overloading is especially useful for repeated state
 * detection.
 *
 * @param n1 a <code>LightsOutNode</code> value - first node
 * @param n2 a <code>LightsOutNode</code> value - second node
 * @return a <code>bool</code> value - whether or not the two nodes are equal
 */
bool operator==(LightsOutNode n1, LightsOutNode n2);

#define LIGHTS_OUT_NODE_H
#endif
