#include "PegSolitaireNode.h"

/**
 * PegSolitaireNode.cc - Traditional 5-on-a-side Triangle Peg
 * Solitaire; Goal: to leave one peg after removal of all others via
 * linear jumps.
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */


/**
 * Creates a <code>PegSolitaireNode</code> instance and sets it
 * to an initial search state. */
PegSolitaireNode::PegSolitaireNode() 
{
  pegs = new char[NUM_PEGS];
  for (int i=0; i<NUM_PEGS; i++)
    pegs[i] = 'O';
  pegs[4] = ' ';
}


/**
 * Destructor freeing dynamically allocated array.
 */
PegSolitaireNode::~PegSolitaireNode() {
  delete [] pegs;
}


/**
 * <code>isGoal</code> - test whether or not the current node is a
 * goal node.
 *
 * @return a <code>bool</code> value */
bool PegSolitaireNode::isGoal() 
{
  // If depth is 13, we've jumped and removed 13 pegs.  Starting
  // with 14, that leaves 1 (our goal).
  return (depth == 13);
}


/**
 * <code>expand</code> - return a (possibly empty) Vector of this
 * node's children
 *
 * @return a <code>Vector</code> of SearchNodes */
vector<SearchNode *> *PegSolitaireNode::expand() 
{
  /*
Peg position layout:
        0 
       / \
      1---2
     / \ / \ 
    3---4---5
   / \ / \ / \
  6---7---8---9 
 / \ / \ / \ / \
10--11--12--13--14
  */

  /**
   * constant <code>LINEAR_POS</code> - lists of positions in a line
   * along which jumps can occur */
  const int LINEAR_POS[9][6] 
  = {{0,1,3,6,10,-1}, {2,4,7,11,-1,-1}, {5,8,12,-1,-1,-1}, {10,11,12,13,14,-1},
     {6,7,8,9,-1,-1}, {3,4,5,-1,-1,-1}, {14,9,5,2,0,-1}, {13,8,4,1,-1,-1}, 
     {12,7,3,-1,-1,-1}};    


  // For each operation, check if it is valid and will change
  // the state.  If so, generate a child and add it to the
  // children vector.
  vector<SearchNode *> *children = new vector<SearchNode *>;
  // For each line of positions, and ...
  for (int i = 0; i < 9; i++) {
    // ... for each three positions along that line
    for (int j = 0; LINEAR_POS[i][j+2] != -1; j++)
      // ... check if a jump can occur.
      if (pegs[LINEAR_POS[i][j+1]] == 'O' 
	  && (pegs[LINEAR_POS[i][j]] != pegs[LINEAR_POS[i][j+2]])) {
	// If it can, create the child node and add it.
	PegSolitaireNode *child = (PegSolitaireNode *) childClone();
	for (int k = j; k < j + 3; k++)
	  child->pegs[LINEAR_POS[i][k]] = 
	    (child->pegs[LINEAR_POS[i][k]] == 'O') ? ' ' : 'O';
	children->push_back(child);
      }
  }
  return children;
}


/**
 * <code>deepCopy</code> - return a deep copy of this node.
 *
 * @return an <code>SearchNode *</code> value
 */
SearchNode *PegSolitaireNode::deepCopy()
{
  PegSolitaireNode *newNode = new PegSolitaireNode(*this);
  newNode->pegs = new char[NUM_PEGS];
  for (int i = 0; i < NUM_PEGS; i++) 
    newNode->pegs[i] = pegs[i];
  return newNode;
}


char PegSolitaireNode::getPeg(int i) 
{
  return pegs[i];
}


/**
 * <code>toStream</code> - print a text representation of this search
 * node to the given output stream. */
ostream& PegSolitaireNode::toStream(ostream &sout)
{
  sout << "        " << pegs[0] << "\n       / \\\n      " 
       << pegs[1] << "---" << pegs[2] << "\n     / \\ / \\ \n    " 
       << pegs[3] << "---" << pegs[4] << "---" << pegs[5] 
       << "\n   / \\ / \\ / \\\n  " << pegs[6] << "---"  << pegs[7] 
       << "---" << pegs[8] << "---" << pegs[9] 
       << " \n / \\ / \\ / \\ / \\\n" << pegs[10] << "---" 
       << pegs[11] << "---" << pegs[12] << "---" << pegs[13]
       << "---" << pegs[14] << '\n';
  return sout;
}


/**
 * == - Equality overloading is especially useful for repeated state
 * detection.
 *
 * @param n1 a <code>PegSolitaireNode</code> value - first node
 * @param n2 a <code>PegSolitaireNode</code> value - second node
 * @return a <code>bool</code> value - whether or not the two nodes are equal
 */
bool operator==(PegSolitaireNode n1, PegSolitaireNode n2) 
{
  // Check that all peg positions are identical.
  for (int i = 0; i < PegSolitaireNode::NUM_PEGS; i++)
    if (n1.getPeg(i) != n2.getPeg(i))
      return false;
  return true;
}



/* test code 
int main() 
{
  PegSolitaireNode *n = new PegSolitaireNode();
  cout << n << endl;
  cout << ((n->isGoal()) ? "goal" : "not goal") << endl;
  vector<SearchNode *> *children = n->expand();
  for (int i = 0; i < (int)children->size(); i++) {
    SearchNode *child = children->at(i);
    cout << (PegSolitaireNode *) child << endl;
    cout << ((child->isGoal()) ? "goal" : "not goal");
  }
}

*/
