#ifndef PEG_SOLITAIRE_NODE_H
#include <vector>
#include "SearchNode.h"
using namespace std;

/**
 * PegSolitaireNode.h - Traditional 5-on-a-side Triangle Peg
 * Solitaire; Goal: to leave one peg after removal of all others via
 * linear jumps.
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */

class PegSolitaireNode : public SearchNode {

 public:
  
  /**
   * constant <code>NUM_PEGS</code> - number of pegs */
  static const int NUM_PEGS = 15;


  /**
   * Creates a <code>PegSolitaireNode</code> instance and sets it
   * to an initial search state. */
  PegSolitaireNode();
  

  /**
   * Destructor freeing dynamically allocated array.
   */
  ~PegSolitaireNode();
  

  /**
   * <code>isGoal</code> - test whether or not the current node is a
   * goal node.
   *
   * @return a <code>bool</code> value */
  bool isGoal();
  
  
  /**
   * <code>expand</code> - return a (possibly empty) vector of
   * pointers to this node's children
   *
   * @return a pointer to a <code>vector</code> of
   * <code>SearchNode</code> pointers */
  vector<SearchNode *> *expand();


  /**
   * <code>deepCopy</code> - return a deep copy of this node.
   *
   * @return an <code>SearchNode *</code> value
   */
  SearchNode *deepCopy();


  char getPeg(int i);
  

  /**
   * <code>toStream</code> - print a text representation of this
   * search node to the given output stream. */
  ostream& toStream(ostream &sout);


 private:

  /**
   * variable <code>pegs</code> - state (peg 'O' or empty ' ') of
   * each of the 15 triangle positions numbers from left to right,
   * top to bottom.  */
  char *pegs;
  
}; // PegSolitaireNode


/**
 * == - Equality overloading is especially useful for repeated state
 * detection.
 *
 * @param n1 a <code>PegSolitaireNode</code> value - first node
 * @param n2 a <code>PegSolitaireNode</code> value - second node
 * @return a <code>bool</code> value - whether or not the two nodes are equal
 */
bool operator==(PegSolitaireNode n1, PegSolitaireNode n2);


/**
 * Overloading << is especially useful for debugging, tracing search
 * execution, reporting results, etc.
 *
 * @param sout an <code>ostream&</code> value - output stream
 * @param n a <code>PegSolitaireNode</code> value - reference to
 * node to be printed to output stream
 * @return an <code>ostream&</code> value - same output stream 
 */
ostream& operator<<(ostream &sout, PegSolitaireNode &n);

#define PEG_SOLITAIRE_NODE_H
#endif
