#ifndef SEARCHNODE_H
#include <iostream>
#include <vector>
using namespace std;

/**
 * SearchNode.h - a simple node for uninformed AI search (assuming
 * cost equals depth).
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */

class SearchNode {
  
 public:
  
  /**
   * variable <code>parent</code> - parent search node; null if and
   * only if node is the root of the search tree.  */
  SearchNode *parent;


  /**
   * variable <code>depth</code> - search depth; 0 at the root
   * search node; a child node has its parent node depth + 1. */
  int depth;
    

  /**
   * constructor - extend if your search node needs extra initialization
   */
  SearchNode();
  

  /**
   * default destructor - override if your search node dynamically
   * allocates memory.
   */
  virtual ~SearchNode();
  

  /**
   * <code>deepCopy</code> - return a reference to a DEEP copy of the
   * current node.  That is, any change to the original/cloned node
   * should have no side-effect on the other.  For example, when the
   * node's state is described using an array, you must create a new
   * array for the cloned node and copy the contents of the original
   * node array. 
   * @return a <code>SearchNode *</code) value - a reference to the
   * deep copy of this node
   */
  virtual SearchNode *deepCopy() = 0;
  

  /**
   * <code>isGoal</code> - test whether or not the current node is a
   * goal node.
   *
   * @return a <code>bool</code> value - whether or not the
   * current node is a goal node */
  virtual bool isGoal() = 0;
  
  
  /**
   * <code>expand</code> - return a (possibly empty) vector of this
   * node's children.  A new child is created by calling
   * <code>childClone</code> and appropriately modifying the state
   * of the returned node.
   *
   * @return a <code>vector</code> of pointers to SearchNodes that are
   * children of this node */

  virtual vector<SearchNode *> *expand() = 0; 


  /**
   * <code>childClone</code> - returns a clone of this node that has
   * been made a child of this node and has a depth one greater than
   * this.
   *
   * @return a <code>SearchNode</code> value */
  SearchNode *childClone();

  /**
   * <code>toStream</code> - print a text representation of this
   * search node to the given output stream.  You are encouraged to
   * override this method for subclasses in order to provide more
   * node-specific information. */
  virtual ostream& toStream(ostream &sout);

}; // SearchNode

/**
 * Overloading << is especially useful for debugging, tracing search
 * execution, reporting results, etc.
 *
 * @param sout an <code>ostream&</code> value - output stream
 * @param n a <code>SearchNode</code> value - reference to
 * node to be printed to output stream
 * @return an <code>ostream&</code> value - same output stream 
 */
ostream& operator<<(ostream &sout, SearchNode &n);
ostream& operator<<(ostream &sout, SearchNode *n);

/**
 * NOTE: It is generally beneficial to also overload the << operator
 * to provide a text representation of the node state.  You will want
 * to override the default == operator if you're using a search
 * algorithm with repeated-state checking. */

#define SEARCHNODE_H
#endif
